@@ -45,8 +45,14 @@ class Inferences(object):
         
         for step in range(steps):
 
-            #Dirichlet distribution returns a valid pmf over n_covariates states
-            v_0 = np.random.dirichlet(np.ones(data.n_covariates), size=1)
+            #Approach for selecting initial v matrix:
+            #First time, try a uniform v matrix, assigning equal weight to all covariates
+            #Subsequent times, sample a random pmf using the dirichlet distribution
+            if step == 0:
+                v_0 = np.full(data.n_covariates, 1/data.n_covariates)
+            else:
+                #Dirichlet distribution returns a valid pmf over n_covariates states
+                v_0 = np.random.dirichlet(np.ones(data.n_covariates), size=1)            
 
             #Required to have non negative values
             bnds = tuple((0,1) for _ in range(data.n_covariates))
@@ -130,7 +136,7 @@ class Inferences(object):
         return loss
 
 
-    def in_space_placebo(self):
+    def in_space_placebo(self, n_optim=3):
         '''
         Fits a synthetic control to each of the control units, 
         using the remaining control units as control group
@@ -162,7 +168,7 @@ class Inferences(object):
                             control_placebo_outcome[:data.periods_pre_treatment], 
                             control_placebo_covariates,
                             data,
-                            "in-space", 3)
+                            "in-space", n_optim)
             
             #Compute outcome of best synthetic control
             if self.method == "SC":
@@ -186,7 +192,7 @@ class Inferences(object):
 
         return
 
-    def in_time_placebo(self, placebo_treatment_period):
+    def in_time_placebo(self, placebo_treatment_period, n_optim=5):
         '''
         Fits a synthetic control to the treated unit,
         with a pre-treatment period shorter than the true pre-treatment period,
@@ -220,7 +226,7 @@ class Inferences(object):
         self.optimize(in_time_placebo_treated_outcome, in_time_placebo_treated_covariates,
                         in_time_placebo_control_outcome, in_time_placebo_control_covariates,
                         data,
-                        "in-time", 5)
+                        "in-time", n_optim)
         #Compute placebo outcomes using placebo_w vector from optimize
         placebo_outcome = data.in_time_placebo_w.T @ in_time_placebo_control_outcome_all.T
 
@@ -301,12 +307,15 @@ class Inferences(object):
             pre_ratio_list.append(treated_pre)
 
             #Store in dataframe
-            rmspe_df = pd.DataFrame({"pre_rmspe": pre_ratio_list,
+            rmspe_df = pd.DataFrame({"unit": data.treated_unit,
+                                    "pre_rmspe": pre_ratio_list,
                                     "post_rmspe": post_ratio_list},
-                                    columns=["pre_rmspe", "post_rmspe"])
-            #Compute post/pre rmspe ratio
+                                    columns=["unit", "pre_rmspe", "post_rmspe"])
+            
+            #Compute post/pre rmspe ratio and add to rmspe_df
             rmspe_df["post/pre"] = rmspe_df["post_rmspe"] / rmspe_df["pre_rmspe"]
-
+            
+            #Store it
             data.rmspe_df = rmspe_df
             return
             
@@ -319,15 +328,17 @@ class Inferences(object):
                 pre_ratio_list.append(pre_ratio)
 
             #Store in dataframe
-            rmspe_df = pd.DataFrame({"pre_rmspe": pre_ratio_list,
+            rmspe_df = pd.DataFrame({"unit":  data.control_units,
+                                    "pre_rmspe": pre_ratio_list,
                                     "post_rmspe": post_ratio_list},
-                                    columns=["pre_rmspe", "post_rmspe"])
+                                    columns=["unit", "pre_rmspe", "post_rmspe"])
             
             #Compute post/pre rmspe ratio
             rmspe_df["post/pre"] = rmspe_df["post_rmspe"] / rmspe_df["pre_rmspe"]
             
             #Extend self.original_data.rmspe_df and return
-            data.rmspe_df = pd.concat([data.rmspe_df, rmspe_df], axis=0)
+            rmspe_df = pd.concat([data.rmspe_df, rmspe_df], axis=0)
+            data.rmspe_df = rmspe_df.reset_index(drop=True)
             return
              
 
