@@ -21,7 +21,7 @@ from SyntheticControlMethods.inferences import Inferences
 
 class SynthBase(object):
     
-    def __init__(self, dataset, outcome_var, id_var, time_var, treatment_period, treated_unit, 
+    def __init__(self, dataset, outcome_var, id_var, time_var, treatment_period, treated_unit, control_units,
                 covariates, periods_all, periods_pre_treatment, n_controls, n_covariates,
                 treated_outcome, control_outcome, treated_covariates, control_covariates,
                 treated_outcome_all, control_outcome_all,
@@ -54,6 +54,7 @@ class SynthBase(object):
         self.time = time_var
         self.treatment_period = treatment_period
         self.treated_unit = treated_unit
+        self.control_units = control_units
         self.covariates = covariates
         self.periods_all = periods_all
         self.periods_pre_treatment = periods_pre_treatment
@@ -108,6 +109,7 @@ class SynthBase(object):
 
         ###Post inference quantities
         self.w = w #Can be provided if using Synthetic DID
+        self.weight_df = None
         self.v = None
         self.treatment_effect = treatment_effect #If known
         self.synth_outcome = None
@@ -128,7 +130,6 @@ class SynthBase(object):
         self.placebo_treatment_period = None
         self.placebo_periods_pre_treatment = None
 
-
 class DataProcessor(object):
     
     def _process_input_data(self, dataset, outcome_var, id_var, time_var, treatment_period, treated_unit, **kwargs):
@@ -137,7 +138,7 @@ class DataProcessor(object):
         These are all the data matrices.
         '''
         #All columns not y, id or time must be predictors
-        covariates = [col for col in dataset.columns if col not in [outcome_var, id_var, time_var]]
+        covariates = [col for col in dataset.columns if col not in [id_var, time_var]]
 
         #Extract quantities needed for pre-processing matrices
         #Get number of periods in pre-treatment and total
@@ -147,6 +148,9 @@ class DataProcessor(object):
         n_controls = dataset[id_var].nunique() - 1
         n_covariates = len(covariates)
 
+        #All units that are not the treated unit are controls
+        control_units = dataset.loc[dataset[id_var] != treated_unit][id_var].unique()
+
         ###Get treated unit matrices first###
         treated_outcome_all, treated_outcome, treated_covariates = self._process_treated_data(
             dataset, outcome_var, id_var, time_var, 
@@ -160,6 +164,11 @@ class DataProcessor(object):
             treatment_period, treated_unit, n_controls, 
             periods_all, periods_pre_treatment, covariates
         )
+        
+        #Rescale covariates to be unit variance (helps with optimization)
+        treated_covariates, control_covariates = self._rescale_covariate_variance(treated_covariates,
+                                                                            control_covariates,
+                                                                            n_covariates)
 
         return {
             'dataset': dataset,
@@ -168,6 +177,7 @@ class DataProcessor(object):
             'time_var':time_var,
             'treatment_period':treatment_period,
             'treated_unit':treated_unit,
+            'control_units':control_units,
             'covariates':covariates,
             'periods_all':periods_all,
             'periods_pre_treatment':periods_pre_treatment,
@@ -223,6 +233,22 @@ class DataProcessor(object):
                 set_index(np.arange(len(control_data[covariates])) // periods_pre_treatment).mean(level=0)).T
 
         return control_outcome_all, control_outcome, control_covariates
+
+    def _rescale_covariate_variance(self, treated_covariates, control_covariates, n_covariates):
+        '''Rescale covariates to be unit variance'''
+
+        #Combine control and treated into one big dataframe, over which we will compute variance for each covariate
+        big_dataframe = np.concatenate((treated_covariates, control_covariates), axis=1)
+
+        #Rescale each covariate to have unit variance
+        big_dataframe /= np.apply_along_axis(np.std, 0, big_dataframe)
+
+        #Re-seperate treated and control from big dataframe
+        treated_covariates = big_dataframe[:,0].reshape(1, n_covariates) #First column is treated unit
+        control_covariates = big_dataframe[:,1:] #All other columns are control units
+
+        #Return covariate matices with unit variance
+        return treated_covariates, control_covariates
     
 class Synth(Inferences, Plot, DataProcessor):
 
@@ -274,7 +300,10 @@ class Synth(Inferences, Plot, DataProcessor):
         
         #Compute rmspe_df
         self._pre_post_rmspe_ratios(None, False)
-        
+
+        #Prepare weight_df with unit weights
+        self.original_data.weight_df = pd.DataFrame({"Unit":self.original_data.control_units,
+                                        "Weight":self.original_data.w.T[0]})
 
 class DiffSynth(Inferences, Plot, DataProcessor):
 
@@ -321,7 +350,7 @@ class DiffSynth(Inferences, Plot, DataProcessor):
         '''
         self.method = "DSC"
 
-        #Process original data - will be used in plotting
+        #Process original data - will be used in plotting and summary
         original_checked_input = self._process_input_data(
             dataset, outcome_var, id_var, time_var, treatment_period, treated_unit, **kwargs
         )
@@ -342,6 +371,10 @@ class DiffSynth(Inferences, Plot, DataProcessor):
         #Compute rmspe_df for treated unit Synthetic Control
         self._pre_post_rmspe_ratios(None, False)
 
+        #Prepare weight_df with unit weights
+        self.weights_df = pd.DataFrame({"Unit":self.original_data.control_units,
+                                        "Weight":self.original_data.w.T[0]})
+
     def difference_data(self, dataset, not_diff_cols):
         '''
         Takes an appropriately formatted, unprocessed dataset
